@@ -8,13 +8,6 @@ MIME::Body - the body of a MIME message
 
 =head1 SYNOPSIS
 
-Before reading further, you should see L<MIME::Tools> to make sure that 
-you understand where this module fits into the grand scheme of things.
-Go on, do it now.  I'll wait.
-
-Ready?  Ok...
-
-
 =head2 Obtaining bodies
 
    ### Get the bodyhandle of a MIME::Entity object:
@@ -139,9 +132,10 @@ use vars qw($VERSION);
 
 ### System modules:
 use Carp;
+use IO::Scalar;
 
 ### The package version, both in 1.23 style *and* usable by MakeMaker:
-$VERSION = "5.420_02";
+$VERSION = substr q$Revision: 5.106 $, 10;
 
 
 #------------------------------
@@ -190,7 +184,6 @@ sub as_lines {
     my $self = shift;
     my @lines;
     my $io = $self->open("r") || return ();
-    local $_;
     push @lines, $_ while (defined($_ = $io->getline()));
     $io->close;
     @lines;
@@ -213,14 +206,12 @@ repeated read() calls; your subclass might wish to override this.
 sub as_string {
     my $self = shift;
     my $str = '';
-    my $fh = IO::File->new(\$str, '>:') or croak("Cannot open in-memory file: $!");
-    $self->print($fh);
-    close($fh);
+    my $out = new IO::Scalar \$str;
+    $self->print($out);
     return $str;
 }
 *data = \&as_string;         ### silenty invoke preferred usage
 
-
 #------------------------------
 
 =item binmode [ONOFF]
@@ -240,23 +231,6 @@ sub binmode {
 
 #------------------------------
 
-=item is_encoded [ONOFF]
-
-I<Instance method.>
-If set to yes, no decoding is applied on output. This flag is set
-by MIME::Parser, if the parser runs in decode_bodies(0) mode, so the
-content is handled unmodified.
-
-=cut
-
-sub is_encoded {
-    my ($self, $yesno) = @_;
-    $self->{MB_IsEncoded} = $yesno if (@_ > 1);
-    $self->{MB_IsEncoded};
-}
-
-#------------------------------
-
 =item dup
 
 I<Instance method.>
@@ -328,7 +302,7 @@ sub print {
     my $nread;
 
     ### Get output filehandle, and ensure that it's a printable object:
-    $fh ||= select;
+    $fh = IO::Wrap::wraphandle($fh || select);
 
     ### Write it:
     my $buf = '';
@@ -365,8 +339,8 @@ The following built-in classes are provided:
    class:               data in:        returns:    
    --------------------------------------------------------
    MIME::Body::File     disk file       IO::Handle   
-   MIME::Body::Scalar   scalar          IO::Handle   
-   MIME::Body::InCore   scalar array    IO::Handle   
+   MIME::Body::Scalar   scalar          IO::Scalar  
+   MIME::Body::InCore   scalar array    IO::ScalarArray
 
 =cut
 
@@ -377,8 +351,8 @@ package MIME::Body::File;
 
 =head2 MIME::Body::File
 
-A body class that stores the data in a disk file.  Invoke the
-constructor as:
+A body class that stores the data in a disk file.  
+The I/O handle is a wrapped filehandle.  Invoke the constructor as:
 
     $body = new MIME::Body::File "/path/to/file";
 
@@ -403,10 +377,11 @@ use vars qw(@ISA);
 use strict;
 
 ### System modules:
-use IO::File;
+require FileHandle;
 
 ### Kit modules:
 use MIME::Tools qw(whine);
+use IO::Wrap;
 
 @ISA = qw(MIME::Body);
 
@@ -425,18 +400,19 @@ sub init {
 #------------------------------
 sub open {
     my ($self, $mode) = @_;
-
+    my $IO;
     my $path = $self->path;
-
-    if( $mode ne 'r' && $mode ne 'w' ) {
+    if ($mode eq 'w') {          ### writing
+	$IO = FileHandle->new(">$path") || die "write-open $path: $!";
+    }
+    elsif ($mode eq 'r') {       ### reading
+	$IO = FileHandle->new("<$path") || die "read-open $path: $!";
+    }
+    else {  
 	die "bad mode: '$mode'";
     }
-
-    my $IO = IO::File->new($path, $mode) || die "MIME::Body::File->open $path: $!";
-
-    $IO->binmode() if $self->binmode;
-
-    return $IO;
+    binmode($IO) if $self->binmode;        ### set binary read/write mode?
+    return (IO::Wrap::wraphandle($IO));    ### wrap if old FileHandle class
 }
 
 #------------------------------
@@ -465,7 +441,7 @@ package MIME::Body::Scalar;
 A body class that stores the data in-core, in a simple scalar.
 Invoke the constructor as:
 
-    $body = new MIME::Body::Scalar \$string;
+    $body = new MIME::Body::Scalar \$scalar;
 
 A single scalar argument sets the body to that value, exactly as though
 you'd opened for the body for writing, written the value, 
@@ -480,12 +456,17 @@ elements of that array together:
                                     "Line 2\n",
                                     "Line 3"];
 
+Uses B<IO::Scalar> as the I/O handle.
+
 =cut
 
 use vars qw(@ISA);
 use strict;
 
 use Carp;
+require FileHandle;
+
+use IO::Scalar;
 
 @ISA = qw(MIME::Body);
 
@@ -513,16 +494,7 @@ sub as_string {
 sub open {
     my ($self, $mode) = @_;
     $self->{MBS_Data} = '' if ($mode eq 'w');        ### writing
-
-    if ($mode eq 'w') {
-	    $mode = '>:';
-    } elsif ($mode eq 'r') {
-	    $mode = '<:';
-    } else {
-	    die "bad mode: $mode";
-    }
-
-    return IO::File->new(\($self->{MBS_Data}), $mode);
+    return new IO::Scalar \($self->{MBS_Data});
 }
 
 
@@ -538,14 +510,13 @@ package MIME::Body::InCore;
 A body class that stores the data in-core.
 Invoke the constructor as:
 
-    $body = new MIME::Body::InCore \$string;
-    $body = new MIME::Body::InCore  $string;
-    $body = new MIME::Body::InCore \@stringarray
+    $body = new MIME::Body::InCore \$scalar;
+    $body = new MIME::Body::InCore \@scalararray
 
 A simple scalar argument sets the body to that value, exactly as though
 you'd opened for the body for writing, written the value, 
 and closed the body again:
-
+    
     $body = new MIME::Body::InCore "Line 1\nLine 2\nLine 3";
 
 A single array reference sets the body to the concatenation of all
@@ -555,14 +526,19 @@ scalars that it holds:
                                     "Line 2\n",
                                     "Line 3"];
 
+Uses B<IO::ScalarArray> as the I/O handle.
+
 =cut
 
 use vars qw(@ISA);
 use strict;
 
 use Carp;
+require FileHandle;
 
-@ISA = qw(MIME::Body::Scalar);
+use IO::ScalarArray;
+
+@ISA = qw(MIME::Body);
 
 
 #------------------------------
@@ -571,16 +547,16 @@ use Carp;
 sub init {
     my ($self, $data) = @_;
     if (!defined($data)) {  ### nothing
-	$self->{MBS_Data} = '';
+	$self->{MBC_Data} = [];
     }
     elsif (!ref($data)) {   ### simple scalar
-	$self->{MBS_Data} = $data;
+	$self->{MBC_Data} = [ $data ];
     }
     elsif (ref($data) eq 'SCALAR') {
-	$self->{MBS_Data} = $$data;
+	$self->{MBC_Data} = [ $$data ];
     }
     elsif (ref($data) eq 'ARRAY') {
-	$self->{MBS_Data} = join('', @$data);
+	$self->{MBC_Data} = $data;
     }
     else {
 	croak "I can't handle DATA which is a ".ref($data)."\n";
@@ -588,6 +564,24 @@ sub init {
     $self;
 }
 
+#------------------------------
+# as_string
+#------------------------------
+sub as_string {
+    my $self = shift;
+    return join '', @{$self->{MBC_Data}};
+}
+
+#------------------------------
+# open READWRITE
+#------------------------------
+sub open {
+    my ($self, $mode) = @_;
+    $self->{MBC_Data} = [] if ($mode eq 'w');        ### writing
+    return new IO::ScalarArray $self->{MBC_Data};
+}
+
+
 1;
 __END__
 
@@ -629,10 +623,10 @@ The default inherited method I<will probably not suffice> for these:
 
 =head1 NOTES
 
-One reason I didn't just use IO::Handle objects for message bodies was
-that I wanted a "body" object to be a form of completely encapsulated
-program-persistent storage; that is, I wanted users to be able to write
-code like this...
+One reason I didn't just use FileHandle or IO::Handle objects for message
+bodies was that I wanted a "body" object to be a form of completely
+encapsulated program-persistent storage; that is, I wanted users
+to be able to write code like this...
 
    ### Get body handle from this MIME message, and read its data:
    $body = $entity->bodyhandle;
@@ -657,7 +651,6 @@ if the user application is processing more than one MIME entity at a time.
 =head1 AUTHOR
 
 Eryq (F<eryq@zeegee.com>), ZeeGee Software Inc (F<http://www.zeegee.com>).
-David F. Skoll (dfs@roaringpenguin.com) http://www.roaringpenguin.com
 
 All rights reserved.  This program is free software; you can redistribute 
 it and/or modify it under the same terms as Perl itself.
@@ -669,7 +662,7 @@ to the use of FileHandles.
 
 =head1 VERSION
 
-$Revision$ $Date$
+$Revision: 5.106 $ $Date: 2000/05/23 05:36:18 $
 
 =cut
 
