@@ -10,9 +10,6 @@ MIME::Decoder::UU - decode a "uuencoded" stream
 
 A generic decoder object; see L<MIME::Decoder> for usage.
 
-Also supports a preamble() method to recover text before
-the uuencoded portion of the stream.
-
 
 =head1 DESCRIPTION
 
@@ -36,7 +33,7 @@ it and/or modify it under the same terms as Perl itself.
 
 =head1 VERSION
 
-$Revision$ $Date$
+$Revision: 5.102 $ $Date: 2000/04/20 04:56:45 $
 
 =cut
 
@@ -49,7 +46,7 @@ use MIME::Tools qw(whine);
 @ISA = qw(MIME::Decoder);
 
 # The package version, both in 1.23 style *and* usable by MakeMaker:
-$VERSION = "5.420_02";
+$VERSION = substr q$Revision: 5.102 $, 10;
 
 
 #------------------------------
@@ -59,32 +56,14 @@ $VERSION = "5.420_02";
 sub decode_it {
     my ($self, $in, $out) = @_;
     my ($mode, $file);
-    my @preamble;
 
-    ### Init:
-    $self->{MDU_Preamble} = \@preamble;
-    $self->{MDU_Mode} = undef;
-    $self->{MDU_File} = undef;
-
-    ### Find beginning...
-    local $_;
+    # Find beginning...
     while (defined($_ = $in->getline)) {
-	if (/^begin(.*)/) {        ### found it: now decode it...
-	    my $modefile = $1;
-	    if ($modefile =~ /^(\s+(\d+))?(\s+(.*?\S))?\s*\Z/) {
-		($mode, $file) = ($2, $4);
-	    }
-	    last;                  ### decoded or not, we're done
-	}
-	push @preamble, $_;
+	last if ($mode, $file) = /^begin\s*(\d*)\s*(\S*)/; 
     }
     die("uu decoding: no begin found\n") if !defined($_);      # hit eof!
-
-    ### Store info:
-    $self->{MDU_Mode} = $mode;
-    $self->{MDU_File} = $file;
-
-    ### Decode:
+    
+    # Decode:
     while (defined($_ = $in->getline)) {
 	last if /^end/;
 	next if /[a-z]/;
@@ -113,39 +92,5 @@ sub encode_it {
     1;
 }
 
-#------------------------------
-#
-# last_preamble
-#
-# Return the last preamble as ref to array of lines.
-# Gets reset by decode_it().
-#
-sub last_preamble {
-    my $self = shift;
-    return $self->{MDU_Preamble} || [];
-}
-
-#------------------------------
-#
-# last_mode
-#
-# Return the last mode.
-# Gets reset to undef by decode_it().
-#
-sub last_mode {
-    shift->{MDU_Mode};
-}
-
-#------------------------------
-#
-# last_filename
-#
-# Return the last filename.
-# Gets reset by decode_it().
-#
-sub last_filename {
-    shift->{MDU_File} || [];
-}
-
 #------------------------------
 1;
