@@ -8,12 +8,6 @@ MIME::Entity - class for parsed-and-decoded MIME message
 
 =head1 SYNOPSIS
 
-Before reading further, you should see L<MIME::Tools> to make sure that 
-you understand where this module fits into the grand scheme of things.
-Go on, do it now.  I'll wait.
-
-Ready?  Ok...
-
     ### Create an entity:
     $top = MIME::Entity->build(From    => 'me@myhost.com',
                                To      => 'you@yourhost.com',
@@ -37,7 +31,7 @@ Ready?  Ok...
 A subclass of B<Mail::Internet>.
 
 This package provides a class for representing MIME message entities,
-as specified in RFCs 2045, 2046, 2047, 2048 and 2049.
+as specified in RFC 1521, I<Multipurpose Internet Mail Extensions>.
 
 
 =head1 EXAMPLES
@@ -119,10 +113,10 @@ Copy an entity (headers, parts... everything but external body data):
     ### Get the body, as a MIME::Body;
     $bodyh = $ent->bodyhandle;
     
-    ### Get the intended MIME type (as declared in the header):
+    ### Get the actual MIME type, in the header:
     $type = $ent->mime_type;
-      
-    ### Get the effective MIME type (in case decoding failed):
+
+    ### Get the effective MIME type (for nonstandard encodings):
     $eff_type = $ent->effective_type;
      
     ### Get preamble, parts, and epilogue:
@@ -223,6 +217,7 @@ use vars qw(@ISA $VERSION);
 use strict;
 
 ### System modules:
+use FileHandle;
 use Carp;
 
 ### Other modules:
@@ -234,7 +229,8 @@ use MIME::Tools qw(:config :msgs :utils);
 use MIME::Head;
 use MIME::Body;
 use MIME::Decoder;
-use IO::Lines;
+use IO::Scalar;
+use IO::Wrap;
 
 @ISA = qw(Mail::Internet);
 
@@ -246,7 +242,7 @@ use IO::Lines;
 #------------------------------
 
 ### The package version, both in 1.23 style *and* usable by MakeMaker:
-$VERSION = "5.420_02";
+$VERSION = substr q$Revision: 5.108 $, 10;
 
 ### Boundary counter:
 my $BCount = 0;
@@ -293,10 +289,7 @@ sub known_field {
 # This is used both internally and by MIME::ParserBase, but it is NOT in
 # the public interface!  Do not use it!
 #
-# We generate one containing a "=_", as RFC2045 suggests:
-#    A good strategy is to choose a boundary that includes a character
-#    sequence such as "=_" which can never appear in a quoted-printable
-#    body.  See the definition of multipart messages in RFC 2046.
+# We generate one containing a "=_", as RFC1521 suggests.
 #
 sub make_boundary {
     return "----------=_".scalar(time)."-$$-".$BCount++;
@@ -356,11 +349,12 @@ the end of the array.  (An INDEX of -1 will place the new part at the
 very end of the array, -2 will place it as the penultimate item in the
 array, etc.)  If OFFSET is not given, the new part is added to the end
 of the array.
-I<Thanks to Jason L Tibbitts III for providing support for OFFSET.>
 
 B<Warning:> in general, you only want to attach parts to entities
 with a content-type of C<multipart/*>).
 
+I<Thanks to Jason L Tibbitts III for providing support for OFFSET.>
+
 =cut
 
 sub add_part {
@@ -466,11 +460,11 @@ field (and it sort of looks like one too).
 
 =item Boundary
 
-I<Multipart entities only. Optional.>
-The boundary string.  As per RFC-2046, it must consist only
+I<Multipart entities only. Optional.>  
+The boundary string.  As per RFC-1521, it must consist only
 of the characters C<[0-9a-zA-Z'()+_,-./:=?]> and space (you'll be
 warned, and your boundary will be ignored, if this is not the case).
-If you omit this, a random string will be chosen... which is probably
+If you omit this, a random string will be chosen... which is probably 
 safer.
 
 =item Charset
@@ -484,7 +478,7 @@ I<Single-part entities only. Optional.>
 An alternative to Path (q.v.): the actual data, either as a scalar
 or an array reference (whose elements are joined together to make
 the actual scalar).  The body is opened on the data using 
-MIME::Body::InCore.
+MIME::Body::Scalar.
 
 =item Description
 
@@ -516,11 +510,6 @@ preferred (content-disposition) locations.  If you explicitly want to
 I<avoid> a recommended filename (even when Path is used), supply this 
 as empty or undef.
 
-=item Id
-
-I<Optional.>
-Set the content-id.
-
 =item Path
 
 I<Single-part entities only. Optional.>  
@@ -535,10 +524,10 @@ The default is true.  (NB: look at how C<attach()> uses it.)
 
 =item Type
 
-I<Optional.>
-The basic content-type (C<"text/plain">, etc.).
-If you don't specify it, it defaults to C<"text/plain">
-as per RFC 2045.  I<Do yourself a favor: put it in.>
+I<Optional.>  
+The basic content-type (C<"text/plain">, etc.). 
+If you don't specify it, it defaults to C<"text/plain"> 
+as per RFC-1521.  I<Do yourself a favor: put it in.>
 
 =back
 
@@ -563,7 +552,6 @@ sub build {
     my $desc         = $params{Description};
     my $top          = exists($params{Top}) ? $params{Top} : 1;
     my $disposition  = $params{Disposition} || 'inline';
-    my $id           = $params{Id};
 
     ### Get recommended filename, allowing explicit no-value value:
     my ($path_fname) = (($params{Path}||'') =~ m{([^/]+)\Z});
@@ -585,7 +573,7 @@ sub build {
 		whine "empty string not a legal boundary: I'm ignoring it";
 		$boundary = undef;
 	    }
-	    elsif ($boundary =~ m{[^0-9a-zA-Z_\'\(\)\+\,\.\/\:\=\?\- ]}) {
+	    elsif ($boundary =~ m{[^0-9a-zA-Z\'\(\)\+\_\,\.\/\:\=\?\- ]}) {
 		whine "boundary ignored: illegal characters ($boundary)";
 		$boundary = undef;
 	    }
@@ -600,10 +588,10 @@ sub build {
 	    $self->bodyhandle(new MIME::Body::File $params{Path});
 	}
 	elsif (defined($params{Data})) {
-	    $self->bodyhandle(new MIME::Body::InCore $params{Data});
+	    $self->bodyhandle(new MIME::Body::Scalar $params{Data});
 	}
 	else { 
-	    die "can't build entity: no body, and not multipart\n";
+	    die "can't build entity: no body, and not multipart!\n";
 	}
 
 	### Check whether we need to binmode():   [Steve Kilbane]
@@ -646,13 +634,12 @@ sub build {
     ### Add other MIME fields:
     $head->replace('Content-transfer-encoding', $encoding) if $encoding;
     $head->replace('Content-description', $desc)           if $desc;
-    $head->replace('Content-id', $id)                      if defined($id);
     $head->replace('MIME-Version', '1.0')                  if $top;
 
     ### Add the X-Mailer field, if top level (use default value if not given):
     $top and $head->replace('X-Mailer', 
-			    "MIME-tools ".(MIME::Tools->version).
-			    " (Entity "  .($VERSION).")"); 
+			    "MIME-tools ".(0+MIME::Tools->version).
+			    " (Entity "  .(0+$VERSION).")"); 
 	
     ### Add remaining user-specified fields, if any:
     while (@paramlist) {
@@ -686,7 +673,7 @@ external data in bodyhandles is I<not> copied to new files!
 Changing the data in one entity's data file, or purging that entity, 
 I<will> affect its duplicate.  Entities with in-core data probably need
 not worry.
-'
+
 =cut
 
 sub dup {
@@ -755,17 +742,16 @@ or raw data (via read()).
 =cut
 
 sub body {
-	my ($self, $value) = @_;
-	if (@_ > 1) {      ### setting body line(s)...
-		croak "you cannot use body() to set the encoded contents\n";
-	}
-	else {             ### getting body lines...
-		my $output = '';
-		my $fh = IO::File->new(\$output, '>:') or croak("Cannot open in-memory file: $!");
-		$self->print_body($fh);
-		close($fh);
-		return split("\n", $output);
-	}
+    my ($self, $value) = @_;
+    if (@_ > 1) {      ### setting body line(s)...
+	croak "you cannot use body() to set the encoded contents\n";
+    }
+    else {             ### getting body lines...
+	my $lines = [];
+	my $lh = IO::Lines->new($lines);
+	$self->print_body($lh);
+	return $lines;
+    }
 }
 
 #------------------------------
@@ -773,32 +759,12 @@ sub body {
 =item bodyhandle [VALUE]
 
 I<Instance method.>
-Get or set an abstract object representing the body of the message.
-The body holds the decoded message data.
-
-B<Note that not all entities have bodies!>
-An entity will have either a body or parts: not both.
-This method will I<only> return an object if this entity can 
-have a body; otherwise, it will return undefined. 
-Whether-or-not a given entity can have a body is determined by 
-(1) its content type, and (2) whether-or-not the parser was told to 
-extract nested messages:
-
-    Type:        | Extract nested? | bodyhandle() | parts()
-    -----------------------------------------------------------------------
-    multipart/*  | -               | undef        | 0 or more MIME::Entity
-    message/*    | true            | undef        | 0 or 1 MIME::Entity
-    message/*    | false           | MIME::Body   | empty list
-    (other)      | -               | MIME::Body   | empty list
-
-If C<VALUE> I<is not> given, the current bodyhandle is returned,
-or undef if the entity cannot have a body.
+Get or set an abstract object representing the body.
 
+If C<VALUE> I<is not> given, the current bodyhandle is returned.
 If C<VALUE> I<is> given, the bodyhandle is set to the new value,
 and the previous value is returned.
 
-See L</parts> for more info.
-
 =cut
 
 sub bodyhandle {
@@ -830,7 +796,7 @@ Such an entity, if parsed, would have its effective_type() set to
 C<"application/octet_stream">, although the mime_type() and the contents 
 of the header would remain the same.
 
-If there is no effective type, the method just returns what 
+If there is no known effective type, the method just returns what 
 mime_type() would.
 
 B<Warning:> the effective type is "sticky"; once set, that effective_type()
@@ -892,22 +858,18 @@ sub head {
 =item is_multipart
 
 I<Instance method.>
-Does this entity's effective MIME type indicate that it's a multipart entity?
+Does this entity's MIME type indicate that it's a multipart entity?
 Returns undef (false) if the answer couldn't be determined, 0 (false)
 if it was determined to be false, and true otherwise.
 Note that this says nothing about whether or not parts were extracted.
 
-NOTE: we switched to effective_type so that multiparts with 
-bad or missing boundaries could be coerced to an effective type
-of C<application/x-unparseable-multipart>.
-
-
 =cut
 
+### NOTE: should we use effective_type() instead?
 sub is_multipart {
     my $self = shift;
     $self->head or return undef;        ### no head, so no MIME type!
-    my ($type, $subtype) = split('/', $self->effective_type);
+    my ($type, $subtype) = split('/', $self->head->mime_type);
     (($type eq 'multipart') ? 1 : 0);
 }
 
@@ -921,9 +883,6 @@ associated MIME::Head object.
 If there is no head, returns undef in a scalar context and
 the empty array in a list context.
 
-B<Before you use this,> consider using effective_type() instead,
-especially if you obtained the entity from a MIME::Parser.
-
 =cut
 
 sub mime_type {
@@ -988,8 +947,6 @@ to be "reparsed" into pseudo-multipart entities.  You should read the
 documentation for those options carefully: it I<is> possible for
 a diddled entity to not be multipart, but still have parts attached to it! 
 
-See L</bodyhandle> for a discussion of parts vs. bodies.
-
 =cut
 
 sub parts {
@@ -1000,24 +957,6 @@ sub parts {
 
 #------------------------------
 
-=item parts_DFS
-
-I<Instance method.>
-Return the list of all MIME::Entity objects included in the entity,
-starting with the entity itself, in depth-first-search order.  
-If the entity has no parts, it alone will be returned.
-
-I<Thanks to Xavier Armengou for suggesting this method.>
-
-=cut
-
-sub parts_DFS {
-    my $self = shift;
-    return ($self, map { $_->parts_DFS } $self->parts);
-}
-
-#------------------------------
-
 =item preamble [LINES]
 
 I<Instance method.>
@@ -1058,58 +997,25 @@ sub preamble {
 
 #------------------------------
 
-=item make_multipart [SUBTYPE], OPTSHASH...
+=item make_multipart [SUBTYPE]
 
 I<Instance method.>
 Force the entity to be a multipart, if it isn't already.
-We do this by replacing the original [singlepart] entity with a new
-multipart that has the same non-MIME headers ("From", "Subject", etc.),
-but all-new MIME headers ("Content-type", etc.).  We then create
-a copy of the original singlepart, I<strip out> the non-MIME headers
-from that, and make it a part of the new multipart.  So this:
-
-    From: me
-    To: you
-    Content-type: text/plain
-    Content-length: 12
-    
-    Hello there!
-
-Becomes something like this:
-
-    From: me
-    To: you
-    Content-type: multipart/mixed; boundary="----abc----"
-         
-    ------abc----
-    Content-type: text/plain
-    Content-length: 12
-    
-    Hello there!
-    ------abc------
-
-The actual type of the new top-level multipart will be "multipart/SUBTYPE" 
-(default SUBTYPE is "mixed").
+The content headers from the top-level are bumped down to the demoted part.
+The actual type will be "multipart/SUBTYPE" (default SUBTYPE is "mixed").
 
 Returns 'DONE'    if we really did inflate a singlepart to a multipart.
-Returns 'ALREADY' (and does nothing) if entity is I<already> multipart
-and Force was not chosen.
-
-If OPTSHASH contains Force=>1, then we I<always> bump the top-level's
-content and content-headers down to a subpart of this entity, even if 
-this entity is already a multipart.  This is apparently of use to 
-people who are tweaking messages after parsing them.
+Returns 'ALREADY' (and does nothing) if entity is I<already> multipart.
 
 =cut
 
 sub make_multipart {
-    my ($self, $subtype, %opts) = @_;
+    my ($self, $subtype) = @_;
     my $tag;
     $subtype ||= 'mixed';
-    my $force = $opts{Force};
 
-    ### Trap for simple case: already a multipart?
-    return 'ALREADY' if ($self->is_multipart and !$force); 
+    ### Trap for simple case:
+    return 'ALREADY' if $self->is_multipart;       ### already a multipart?
 
     ### Rip out our guts, and spew them into our future part:
     my $part = bless {%$self}, ref($self);         ### part is a shallow copy
@@ -1194,19 +1100,6 @@ I<Instance method.>
 Recursively purge (e.g., unlink) all external (e.g., on-disk) body parts 
 in this message.  See MIME::Body::purge() for details.
 
-B<Note:> this does I<not> delete the directories that those body parts
-are contained in; only the actual message data files are deleted.
-This is because some parsers may be customized to create intermediate
-directories while others are not, and it's impossible for this class
-to know what directories are safe to remove.  Only your application
-program truly knows that.
-  
-B<If you really want to "clean everything up",> one good way is to
-use C<MIME::Parser::file_under()>, and then do this before parsing
-your next message:
-
-    $parser->filer->purge();
-
 I wouldn't attempt to read those body files after you do this, for
 obvious reasons.  As of MIME-tools 4.x, each body's path I<is> undefined
 after this operation.  I warned you I might do this; truly I did.
@@ -1372,9 +1265,9 @@ sub sign {
 	$sig = (ref($sig) ? join('', @$sig) : $sig);
     }
     elsif ($params{File}) {                      ### file contents
-	my $fh = IO::File->new( $params{File} ) or croak "can't open $params{File}: $!";
-	$sig = join('', $fh->getlines);
-	$fh->close or croak "can't close $params{File}: $!";
+	CORE::open SIG, $params{File} or croak "can't open $params{File}: $!";
+	$sig = join('', SIG->getlines);
+	close SIG;
     }
     else {
 	croak "no signature given!";
@@ -1405,11 +1298,11 @@ sub sign {
 
 	### Output data back into body, followed by signature:
 	my $line;
-	$io = $self->open("w") or croak("open: $!");
+	$io = $self->open("w");
 	foreach $line (@body) { $io->print($line) };      ### body data
 	(($body[-1]||'') =~ /\n\Z/) or $io->print("\n");  ### ensure final \n
 	$io->print("-- \n$sig");                          ### separator + sig
-	$io->close or croak("close: $!");
+	$io->close;	
 	return 1;         ### done!
     }
 }
@@ -1446,8 +1339,8 @@ sub suggest_encoding {
 	$self->bodyhandle || return ($self->parts ? 'binary' : '7bit');
 	my ($IO, $unclean);
 	if ($IO = $self->bodyhandle->open("r")) {
-	    ### Scan message for 7bit-cleanliness
-	    local $_;
+
+	    ### Scan message for 7bit-cleanliness:
 	    while (defined($_ = $IO->getline)) {
 		last if ($unclean = ((length($_) > 999) or /[\200-\377]/));
 	    }
@@ -1593,7 +1486,7 @@ Currently unimplemented for MIME messages.  Does nothing, returns false.
 =cut
 
 sub tidy_body {
-    usage "MIME::Entity::tidy_body currently does nothing";
+    carp "MIME::Entity::tidy_body currently does nothing" if $^W;
     0;
 }
 
@@ -1739,12 +1632,10 @@ some sort of email handler, it's up to you to save this information.
 
 =cut
 
-use Symbol;
 sub print {
     my ($self, $out) = @_;
-    $out = select if @_ < 2;
-    $out = Symbol::qualify($out,scalar(caller)) unless ref($out);
-
+    $out = wraphandle($out || select);             ### get a printable output
+    
     $self->print_header($out);   ### the header
     $out->print("\n");
     $self->print_body($out);     ### the "stuff after the header"
@@ -1785,7 +1676,7 @@ filehandle is given: this may lead to confusion.
 
 sub print_body {
     my ($self, $out) = @_;
-    $out ||= select;
+    $out = wraphandle($out || select);             ### get a printable output
     my ($type) = split '/', lc($self->mime_type);  ### handle by MIME type
 
     ### Multipart...
@@ -1793,24 +1684,18 @@ sub print_body {
 	my $boundary = $self->head->multipart_boundary; 
 
 	### Preamble:
-	my $preamble = join('', @{ $self->preamble || $DefPreamble });
-	$out->print("$preamble\n") if ($preamble ne '' or $self->preamble);
+	$out->print(join('', @{ $self->preamble || $DefPreamble }));
 
 	### Parts:
 	my $part;
 	foreach $part ($self->parts) {
-	    $out->print("--$boundary\n");
+	    $out->print("\n--$boundary\n");
 	    $part->print($out);
-	    $out->print("\n");           ### needed for next delim/close
 	}
-	$out->print("--$boundary--\n");
 
 	### Epilogue:
-	my $epilogue = join('', @{ $self->epilogue || $DefEpilogue });
-	if ($epilogue ne '') {
-	    $out->print($epilogue);
-	    $out->print("\n") if ($epilogue !~ /\n\Z/);  ### be nice
-	}
+	$out->print("\n--$boundary--\n");
+	$out->print(join('', @{ $self->epilogue || $DefEpilogue }));
     }
 
     ### Singlepart type with parts...
@@ -1822,7 +1707,7 @@ sub print_body {
 	foreach $part ($self->parts) {
 	    $out->print("\n\n") if $need_sep++;
 	    $part->print($out);
-	}
+	}	
     }
 
     ### Singlepart type, or no parts: output body...
@@ -1844,22 +1729,16 @@ sub print_body {
 #
 sub print_bodyhandle {
     my ($self, $out) = @_;
-    $out ||= select;
+    $out = wraphandle($out || select);             ### get a printable output
 
+    ### Get the encoding, defaulting to "binary" if unsupported:
+    my $encoding = ($self->head->mime_encoding || 'binary');
+    my $decoder = best MIME::Decoder $encoding;
+    $decoder->head($self->head);      ### associate with head, if any
+
+    ### Output the body:
     my $IO = $self->open("r")     || die "open body: $!";
-    if ( $self->bodyhandle->is_encoded ) {
-      ### Transparent mode: data is already encoded, so no
-      ### need to encode it again
-      my $buf;
-      $out->print($buf) while ($IO->read($buf, 2048));
-    } else {
-      ### Get the encoding, defaulting to "binary" if unsupported:
-      my $encoding = ($self->head->mime_encoding || 'binary');
-      my $decoder = best MIME::Decoder $encoding;
-      $decoder->head($self->head);      ### associate with head, if any
-      $decoder->encode($IO, $out, textual_type($self->head->mime_type) ? 1 : 0)   || return error "encoding failed";
-    }
-
+    $decoder->encode($IO, $out)   || return error "encoding failed";
     $IO->close;
     1;
 }
@@ -1881,21 +1760,17 @@ the OUTSTREAM.
 =item stringify
 
 I<Instance method.>
-Return the entity as a string, exactly as C<print> would print it. 
-The body will be encoded as necessary, and will contain any subparts.  
-You can also use C<as_string()>.
+Return the entity as a string, exactly as C<print> would print it: 
+the body will be encoded as necessary, and will contain any subparts.  
+You can also use C<as_string>.
 
 =cut
 
 sub stringify {
-	my ($self) = @_;
-	my $output = '';
-	my $fh = IO::File->new( \$output, '>:' ) or croak("Cannot open in-memory file: $!");
-	$self->print($fh);
-	$fh->close;
-	return $output;
+    my $str = '';
+    shift->print(new IO::Scalar \$str);
+    $str;    
 }
-
 sub as_string { shift->stringify };      ### silent BC
 
 #------------------------------
@@ -1903,39 +1778,41 @@ sub as_string { shift->stringify };      ### silent BC
 =item stringify_body
 
 I<Instance method.>
-Return the I<encoded> message body as a string, exactly as C<print_body> 
-would print it.  You can also use C<body_as_string()>.
+Return the body as a string, exactly as C<print_body> would print it: 
+the body will be encoded as necessary, and will I<not> contain any subparts.  
+You can also use C<body_as_string>.
 
-If you want the I<unencoded> body, and you are dealing with a
-singlepart message (like a "text/plain"), use C<bodyhandle()> instead:
-
-    if ($ent->bodyhandle) {
-	$unencoded_data = $ent->bodyhandle->as_string;
-    }
-    else {
-	### this message has no body data (but it might have parts!)
-    }
+If you want the I<unencoded> body, use C<$ent->body->as_string>.
 
 =cut
 
 sub stringify_body {
-	my ($self) = @_;
-	my $output = '';
-	my $fh = IO::File->new( \$output, '>:' ) or croak("Cannot open in-memory file: $!");
-	$self->print_body($fh);
-	$fh->close;
-	return $output;
+    my $str = '';
+    shift->print_body(new IO::Scalar \$str);
+    $str;    
 }
-
 sub body_as_string { shift->stringify_body }
 
+#------------------------------
+#
+# stringify_bodyhandle
+#
+# Instance method, unpublicized.  Stringify just the bodyhandle.
+
+sub stringify_bodyhandle {
+    my $str = '';
+    shift->print_bodyhandle(new IO::Scalar \$str);
+    $str;    
+}
+sub bodyhandle_as_string { shift->stringify_bodyhandle }
+
 #------------------------------
 
 =item stringify_header 
 
 I<Instance method.>
 Return the header as a string, exactly as C<print_header> would print it.
-You can also use C<header_as_string()>.
+You can also use C<header_as_string>.
 
 =cut
 
@@ -1945,8 +1822,9 @@ sub stringify_header {
 sub header_as_string { shift->stringify_header }
 
 
-1;
+
 __END__
+1;
   
 #------------------------------
 
@@ -1967,25 +1845,20 @@ containing the header information.
 
 =item *
 
-A I<bodyhandle>, which is a reference to a MIME::Body object
-containing the decoded body data.  This is only defined if 
-the message is a "singlepart" type:
-
-    application/*
-    audio/*
-    image/*
-    text/*
-    video/*
+A I<bodyhandle>, which is a reference a MIME::Body object
+containing the decoded body data.
+(In pre-2.0 releases, this was accessed via I<body>, 
+which was a path to a file containing the decoded body.
+Integration with Mail::Internet has forced this to change.)
 
 =item *
 
-An array of I<parts>, where each part is a MIME::Entity object.  
-The number of parts will only be nonzero if the content-type 
-is I<not> one of the "singlepart" types:
-
-    message/*        (should have exactly one part)
-    multipart/*      (should have one or more parts)
+A list of zero or more I<parts>, each of which is a MIME::Entity 
+object.  The number of parts will only be nonzero if the content-type 
+is some subtype of C<"multipart">.
 
+Note that a multipart entity does I<not> have a body.
+Of course, any/all of its component parts can have bodies.
 
 =back
 
@@ -2000,65 +1873,36 @@ out there.  But let's open up the floor for a few questions...
 
 =over 4
 
-=item What is the difference between a "message" and an "entity"?
 
-A B<message> is the actual data being sent or received; usually
-this means a stream of newline-terminated lines.
-An B<entity> is the representation of a message as an object.
-
-This means that you get a "message" when you print an "entity" 
-I<to> a filehandle, and you get an "entity" when you parse a message
-I<from> a filehandle.
-
-
-=item What is a message body?
-
-B<Mail::Internet:> 
-The portion of the printed message after the header.
-
-B<MIME::Entity:>
-The portion of the printed message after the header.
-
-
-=item How is a message body stored in an entity?
+=item How are message bodies stored?
 
 B<Mail::Internet:> 
 As an array of lines.
 
 B<MIME::Entity:> 
-It depends on the content-type of the message.
-For "container" types (C<multipart/*>, C<message/*>), we store the
-contained entities as an array of "parts", accessed via the C<parts()>
-method, where each part is a complete MIME::Entity.
-For "singlepart" types (C<text/*>, C<image/*>, etc.), the unencoded
-body data is referenced via a MIME::Body object, accessed via 
-the C<bodyhandle()> method:
-
-                      bodyhandle()   parts()
-    Content-type:     returns:       returns:
-    ------------------------------------------------------------
-    application/*     MIME::Body     empty
-    audio/*           MIME::Body     empty     
-    image/*           MIME::Body     empty      
-    message/*         undef          MIME::Entity list (usually 1)
-    multipart/*       undef          MIME::Entity list (usually >0)
-    text/*            MIME::Body     empty     
-    video/*           MIME::Body     empty     
-    x-*/*             MIME::Body     empty 
-
-As a special case, C<message/*> is currently ambiguous: depending 
-on the parser, a C<message/*> might be treated as a singlepart,
-with a MIME::Body and no parts.  Use bodyhandle() as the final 
-arbiter.
+As a MIME::Body object, where the data may reside on disk or 
+in-core, may be large, and may be binary (not line-oriented).
 
 
-=item What does the body() method return?
+=item Do messages generally have bodies?
 
 B<Mail::Internet:> 
-As an array of lines, ready for sending.
+Almost certainly yes.
+
+B<MIME::Entity:>   
+Yes if this is a I<singlepart> message, and NO if it's a 
+I<multipart> message... since for multiparts the message "body" is 
+stored as the parsed collection of "parts" (each of which
+is also a MIME::Entity).
+
+
+=item If an entity has a body, does it have a soul as well?
+
+The soul does not exist in a corporeal sense, the way the body does; 
+it is not a solid [Perl] object.  Rather, it is a virtual object
+which is only visible when you print() an entity to a file... in other
+words, the "soul" it is all that is left after the body is DESTROY'ed.  
 
-B<MIME::Entity:> 
-As an array of lines, ready for sending.
 
 =item What's the best way to get at the body data?
 
@@ -2066,64 +1910,24 @@ B<Mail::Internet:>
 Use the body() method.
 
 B<MIME::Entity:> 
-Depends on what you want... the I<encoded> data (as it is 
-transported), or the I<unencoded> data?  Keep reading...
-
-
-=item How do I get the "encoded" body data?
-
-B<Mail::Internet:> 
-Use the body() method.
-
-B<MIME::Entity:> 
-Use the body() method.  You can also use:
-
-    $entity->print_body()
-    $entity->stringify_body()   ### a.k.a. $entity->body_as_string()
-
-
-=item How do I get the "unencoded" body data?
-
-B<Mail::Internet:> 
-Use the body() method.
-
-B<MIME::Entity:> 
-Use the I<bodyhandle()> method!
-If bodyhandle() method returns true, then that value is a 
-L<MIME::Body|MIME::Body> which can be used to access the data via 
-its open() method.  If bodyhandle() method returns an undefined value, 
-then the entity is probably a "container" that has no real body data of
-its own (e.g., a "multipart" message): in this case, you should access
-the components via the parts() method.  Like this:
-
-    if ($bh = $entity->bodyhandle) {
-	$io = $bh->open;
-	...access unencoded data via $io->getline or $io->read...
-	$io->close;
-    }
-    else {
-	foreach my $part (@parts) {
-	    ...do something with the part...
-	}
-    }
-
-You can also use:
-
-    if ($bh = $entity->bodyhandle) {
-	$unencoded_data = $bh->as_string;
-    }
-    else {
-	...do stuff with the parts...
-    }
+Use the bodyhandle() method, or the brand-new open()
+method.  The open() method returns a filehandle-like object to
+you, which gives you methods like getline() and read().  
+I<Use methods only> for portability; don't make any assumptions
+about what you've been handed.
 
 
 =item What does the body() method return?
 
 B<Mail::Internet:> 
-The transport-encoded message body, as an array of lines.
+The body, as an array of lines.
 
 B<MIME::Entity:>   
-The transport-encoded message body, as an array of lines.
+The body, as an array of lines...
+but I<only> for a singlepart messages.  It returns I<nothing> for 
+a multipart message, since multiparts by definition do not have bodies
+of their own.  It's also somewhat inappropriate for non-textual 
+bodies, like GIFs.
 
 
 =item What does print_body() print?
@@ -2132,7 +1936,37 @@ B<Mail::Internet:>
 Exactly what body() would return to you.
 
 B<MIME::Entity:> 
-Exactly what body() would return to you.
+The I<encoded representation> of "all the stuff following the header",
+using the Content-transfer-encoding in the MIME header. This includes
+the encoded representation of any I<parts> as well.
+Put simply, print_body() doesn't just "print the body": it
+prints a flattened representation of the I<entire entity,>
+including subparts.  This is generally what people seem to expect.
+
+
+=item Assuming I have a singlepart, isn't the data 
+      from body() identical to the stuff printed by print_body()?
+
+B<Mail::Internet:> 
+Yes.
+
+B<MIME::Entity:>   
+Not likely.  If the original message held a base64-encoded GIF file, 
+the body() data will be the I<actual, decoded, binary GIF data>... 
+which is I<not> the same as that base64-encoded
+stream of ASCII output by print_body().
+
+
+=item Conceptually, what's the difference between what's returned
+      by body() and what's printed by print_body()?
+
+B<Mail::Internet:> 
+None.
+
+B<MIME::Entity:> 
+Method body() refers to the I<actual body data> of the entity in question.
+Method print_body() (and stringify_body()) refers to the 
+I<complete printed representation> of that entity.
 
 
 =item Say I have an entity which might be either singlepart or multipart.
@@ -2169,6 +2003,18 @@ mail message... one whose internal data you don't care much about.
 MIME::Entity, in contrast, cares I<very much> about the message contents: 
 that's its job!
 
+Here's an example:
+
+Suppose you wanted me to rewrite MIME::Entity so that you could properly
+set any body -- even a multipart body -- by giving its lines to body().  
+After all, I can just parse the lines you give me, right?
+
+Not quite.  In order to parse that data, I I<need> to have a header
+which tells me whether it's singlepart or multipart.  I I<need>
+to know the encoding, too.  So MIME::Entity will enforces a sequence 
+of events on how you set things up, unlike Mail::Internet -- which 
+doesn't care about message contents.
+
 =back
 
 
@@ -2183,30 +2029,30 @@ In multipart messages, the I<"preamble"> is the portion that precedes
 the first encapsulation boundary, and the I<"epilogue"> is the portion
 that follows the last encapsulation boundary.
 
-According to RFC 2046:
+According to RFC-1521:
 
-    There appears to be room for additional information prior
-    to the first encapsulation boundary and following the final
+    There appears to be room for additional information prior 
+    to the first encapsulation boundary and following the final 
     boundary.  These areas should generally be left blank, and
-    implementations must ignore anything that appears before the
+    implementations must ignore anything that appears before the 
     first boundary or after the last one.
 
-    NOTE: These "preamble" and "epilogue" areas are generally
-    not used because of the lack of proper typing of these parts
-    and the lack of clear semantics for handling these areas at
-    gateways, particularly X.400 gateways.  However, rather than
-    leaving the preamble area blank, many MIME implementations
-    have found this to be a convenient place to insert an
-    explanatory note for recipients who read the message with
-    pre-MIME software, since such notes will be ignored by
+    NOTE: These "preamble" and "epilogue" areas are generally 
+    not used because of the lack of proper typing of these parts 
+    and the lack of clear semantics for handling these areas at 
+    gateways, particularly X.400 gateways.  However, rather than 
+    leaving the preamble area blank, many MIME implementations 
+    have found this to be a convenient place to insert an 
+    explanatory note for recipients who read the message with 
+    pre-MIME software, since such notes will be ignored by 
     MIME-compliant software.
 
-In the world of standards-and-practices, that's the standard.
-Now for the practice:
+In the world of standards-and-practices, that's the standard.  
+Now for the practice: 
 
 I<Some "MIME" mailers may incorrectly put a "part" in the preamble>.
 Since we have to parse over the stuff I<anyway>, in the future I
-I<may> allow the parser option of creating special MIME::Entity objects
+I<may> allow the parser option of creating special MIME::Entity objects 
 for the preamble and epilogue, with bogus MIME::Head objects.
 
 For now, though, we're MIME-compliant, so I probably won't change
@@ -2218,7 +2064,6 @@ how we work.
 =head1 AUTHOR
 
 Eryq (F<eryq@zeegee.com>), ZeeGee Software Inc (F<http://www.zeegee.com>).
-David F. Skoll (dfs@roaringpenguin.com) http://www.roaringpenguin.com
 
 All rights reserved.  This program is free software; you can redistribute 
 it and/or modify it under the same terms as Perl itself.
@@ -2226,8 +2071,9 @@ it and/or modify it under the same terms as Perl itself.
 
 =head1 VERSION
 
-$Revision$ $Date$
+$Revision: 5.108 $ $Date: 2000/05/24 05:12:52 $
 
 =cut
 
 #------------------------------
+1;
