@@ -8,11 +8,6 @@ MIME::Head - MIME message header (a subclass of Mail::Header)
 
 =head1 SYNOPSIS
 
-Before reading further, you should see L<MIME::Tools> to make sure that 
-you understand where this module fits into the grand scheme of things.
-Go on, do it now.  I'll wait.
-
-Ready?  Ok...
 
 =head2 Construction
 
@@ -69,7 +64,7 @@ Ready?  Ok...
     ### Get rid of internal newlines in fields:
     $head->unfold;
     
-    ### Decode any Q- or B-encoded-text in fields (DEPRECATED):
+    ### Decode any Q- or B-encoded-text in fields:
     $head->decode;
      
 
@@ -95,10 +90,9 @@ Ready?  Ok...
 
 =head1 DESCRIPTION
 
-A class for parsing in and manipulating RFC-822 message headers, with
-some methods geared towards standard (and not so standard) MIME fields
-as specified in the various I<Multipurpose Internet Mail Extensions>
-RFCs (starting with RFC 2045)
+A class for parsing in and manipulating RFC-822 message headers, with some 
+methods geared towards standard (and not so standard) MIME fields as 
+specified in RFC-1521, I<Multipurpose Internet Mail Extensions>.
 
 
 =head1 PUBLIC INTERFACE
@@ -114,6 +108,8 @@ use strict;
 use vars qw($VERSION @ISA @EXPORT_OK);
 
 ### System modules:
+use Carp;
+use IO::Wrap;
 
 ### Other modules:
 use Mail::Header 1.09 ();
@@ -130,6 +126,7 @@ use MIME::Field::ContType;
 @ISA = qw(Mail::Header);
 
 
+
 #------------------------------
 #
 # Public globals...
@@ -137,7 +134,7 @@ use MIME::Field::ContType;
 #------------------------------
 
 ### The package version, both in 1.23 style *and* usable by MakeMaker:
-$VERSION = "5.420_02";
+$VERSION = substr q$Revision: 5.105 $, 10;
 
 ### Sanity (we put this test after our own version, for CPAN::):
 use Mail::Header 1.06 ();
@@ -203,10 +200,10 @@ sub from_file {
     my $class = ref($self) ? ref($self) : $self;
 
     ### Parse:
-    my $fh = IO::File->new($file, '<') or return error("open $file: $!");
-    $fh->binmode() or return error("binmode $file: $!");  # we expect to have \r\n at line ends, and want to keep 'em.
-    $self = $class->new($fh, @opts);      ### now, $self is instance or undef
-    $fh->close or return error("close $file: $!");
+    open(HDR, $file) or return error("open $file: $!");
+    binmode(HDR);  # we expect to have \r\n at line ends, and want to keep 'em.
+    $self = $class->new(\*HDR, @opts);      ### now, $self is instance or undef
+    close(HDR);
     $self;
 }
 
@@ -321,30 +318,15 @@ and some true value if it does.
 
 #------------------------------
 
-=item decode [FORCE]
+=item decode
 
-I<Instance method, DEPRECATED.>
-Go through all the header fields, looking for RFC 1522 / RFC 2047 style
-"Q" (quoted-printable, sort of) or "B" (base64) encoding, and decode
-them in-place.  Fellow Americans, you probably don't know what the hell
-I'm talking about.  Europeans, Russians, et al, you probably do.
-C<:-)>.
+I<Instance method.>
+Go through all the header fields, looking for RFC-1522-style "Q"
+(quoted-printable, sort of) or "B" (base64) encoding, and decode them
+in-place.  Fellow Americans, you probably don't know what the hell I'm
+talking about.  Europeans, Russians, et al, you probably do.  C<:-)>. 
 
-B<This method has been deprecated.>
-See L<MIME::Parser/decode_headers> for the full reasons.
-If you absolutely must use it and don't like the warning, then
-provide a FORCE:
-
-   "I_NEED_TO_FIX_THIS"
-          Just shut up and do it.  Not recommended.
-          Provided only for those who need to keep old scripts functioning.
-
-   "I_KNOW_WHAT_I_AM_DOING"
-          Just shut up and do it.  Not recommended.
-          Provided for those who REALLY know what they are doing.
-
-B<What this method does.>
-For an example, let's consider a valid email header you might get:
+For example, here's a valid header you might get:
 
     From: =?US-ASCII?Q?Keith_Moore?= <moore@cs.utk.edu>
     To: =?ISO-8859-1?Q?Keld_J=F8rn_Simonsen?= <keld@dkuug.dk>
@@ -365,7 +347,7 @@ B<Note:> currently, the decodings are done without regard to the
 character set: thus, the Q-encoding C<=F8> is simply translated to the
 octet (hexadecimal C<F8>), period.  For piece-by-piece decoding
 of a given field, you want the array context of 
-C<MIME::Word::decode_mimewords()>.
+L<MIME::Word::decode_mimewords()>.
 
 B<Warning:> the CRLF+SPACE separator that splits up long encoded words 
 into shorter sequences (see the Subject: example above) gets lost
@@ -382,14 +364,6 @@ RFC-1522-decoding code.>
 
 sub decode {
     my $self = shift;
-
-    ### Warn if necessary:
-    my $force = shift || 0;
-    unless (($force eq "I_NEED_TO_FIX_THIS") ||
-	    ($force eq "I_KNOW_WHAT_I_AM_DOING")) {
-	usage "decode is deprecated for safety";
-    }
-
     my ($tag, $i, @decoded);
     foreach $tag ($self->tags) {
 	@decoded = map { scalar(decode_mimewords($_, Field=>$tag))
@@ -398,7 +372,6 @@ sub decode {
 	    $self->replace($tag, $decoded[$i], $i);
 	}
     }
-    $self->{MH_Decoded} = 1;
     $self;
 }
 
@@ -527,7 +500,7 @@ Also, it defaults to the I<currently-selected> filehandle if none is given
 
 sub print {
     my ($self, $fh) = @_;
-    $fh ||= select;
+    $fh = wraphandle($fh || select);   ### get output handle, as a print()able
     $fh->print($self->as_string);
 }
 
@@ -680,22 +653,17 @@ I<Instance method.>
 Try I<real hard> to determine the content transfer encoding
 (e.g., C<"base64">, C<"binary">), which is returned in all-lowercase.
 
-If no encoding could be found, the default of C<"7bit"> is returned
-I quote from RFC 2045 section 6.1:
+If no encoding could be found, the default of C<"7bit"> is returned.  
+I quote from RFC-1521 section 5:
 
-    This is the default value -- that is, "Content-Transfer-Encoding: 7BIT"
+    This is the default value -- that is, "Content-Transfer-Encoding: 7BIT" 
     is assumed if the Content-Transfer-Encoding header field is not present.
 
-I do one other form of fixup: "7_bit", "7-bit", and "7 bit" are
-corrected to "7bit"; likewise for "8bit".
-
 =cut
 
 sub mime_encoding {
     my $self = shift;
-    my $enc = lc($self->mime_attr('content-transfer-encoding') || '7bit');
-    $enc =~ s{^([78])[ _-]bit\Z}{$1bit};
-    $enc;
+    lc($self->mime_attr('content-transfer-encoding') || '7bit');
 }
 
 #------------------------------
@@ -704,22 +672,16 @@ sub mime_encoding {
 
 I<Instance method.>
 Try C<real hard> to determine the content type (e.g., C<"text/plain">,
-C<"image/gif">, C<"x-weird-type">, which is returned in all-lowercase.
-"Real hard" means that if no content type could be found, the default
-(usually C<"text/plain">) is returned.  From RFC 2045 section 5.2:
+C<"image/gif">, C<"x-weird-type">, which is returned in all-lowercase.  
+"Real hard" means that if no content type could be found, the default 
+(usually C<"text/plain">) is returned.  From RFC-1521 section 7.1:
 
-   Default RFC 822 messages without a MIME Content-Type header are
-   taken by this protocol to be plain text in the US-ASCII character
-   set, which can be explicitly specified as:
+    The default Content-Type for Internet mail is 
+    "text/plain; charset=us-ascii".
 
-      Content-type: text/plain; charset=us-ascii
-
-   This default is assumed if no Content-Type header field is specified.
-
-Unless this is a part of a "multipart/digest", in which case
-"message/rfc822" is the default.  Note that you can also I<set> the
-default, but you shouldn't: normally only the MIME parser uses this
-feature.
+Unless this is a part of a "message/digest", in which case "message/rfc822"
+is the default.  Note that you can also I<set> the default, but you 
+shouldn't: normally only the MIME parser uses this feature.
 
 =cut
 
@@ -736,26 +698,28 @@ sub mime_type {
 =item multipart_boundary
 
 I<Instance method.>
-If this is a header for a multipart message, return the
+If this is a header for a multipart message, return the 
 "encapsulation boundary" used to separate the parts.  The boundary
 is returned exactly as given in the C<Content-type:> field; that
 is, the leading double-hyphen (C<-->) is I<not> prepended.
 
-Well, I<almost> exactly... this passage from RFC 2046 dictates
+Well, I<almost> exactly... this passage from RFC-1521 dictates
 that we remove any trailing spaces:
 
-   If a boundary appears to end with white space, the white space
+   If a boundary appears to end with white space, the white space 
    must be presumed to have been added by a gateway, and must be deleted.
 
 Returns undef (B<not> the empty string) if either the message is not
-multipart or if there is no specified boundary.
+multipart, if there is no specified boundary, or if the boundary is
+illegal (e.g., if it is empty after all trailing whitespace has been
+removed).
 
 =cut
 
 sub multipart_boundary {
     my $self = shift;
     my $value =  $self->mime_attr('content-type.boundary');
-    (!defined($value)) ? undef : $value;
+    (!defined($value) or $value eq '') ? undef : $value;
 }
 
 #------------------------------
@@ -770,24 +734,23 @@ Returns undef if no filename could be suggested.
 
 =cut
 
-sub recommended_filename 
-{
-	my $self = shift;
+sub recommended_filename {
+    my $self = shift;
+    my $value;
 
-	# Try these headers in order, taking the first defined,
-	# non-blank one we find.
-	foreach my $attr_name ( qw( content-disposition.filename content-type.name ) ) {
-		my $value = $self->mime_attr( $attr_name );
-		if ( defined $value 
-		    && $value ne ''
-		    && $value =~ /\S/ ) {
-			return $value;
-		}
-	}
+    ### Start by trying to get 'filename' from the 'content-disposition':
+    $value = $self->mime_attr('content-disposition.filename');
+    return $value if (defined($value) and $value ne '');
 
-	return undef;
+    ### No?  Okay, try to get 'name' from the 'content-type':
+    $value = $self->mime_attr('content-type.name');
+    return $value if (defined($value) and $value ne '');
+
+    ### Sorry:
+    undef;
 }
 
+
 #------------------------------
 
 =back
@@ -829,16 +792,18 @@ I quote from Achim Bohnet, who gave feedback on v.1.9 (I think
 he's using the word "header" where I would use "field"; e.g.,
 to refer to "Subject:", "Content-type:", etc.):
 
-    There is also IMHO no requirement [for] MIME::Heads to look
-    like [email] headers; so to speak, the MIME::Head [simply stores]
+    There is also IMHO no requirement [for] MIME::Heads to look 
+    like [email] headers; so to speak, the MIME::Head [simply stores] 
     the attributes of a complex object, e.g.:
 
         new MIME::Head type => "text/plain",
                        charset => ...,
                        disposition => ..., ... ;
 
-I agree in principle, but (alas and dammit) RFC 2045 says otherwise.
-RFC 2045 [MIME] headers are a syntactic subset of RFC-822 [email] headers.
+I agree in principle, but (alas and dammit) RFC-1521 says otherwise.
+RFC-1521 [MIME] headers are a syntactic subset of RFC-822 [email] headers.
+Perhaps a better name for these modules would be RFC1521:: instead of
+MIME::, but we're a little beyond that stage now.
 
 In my mind's eye, I see an abstract class, call it MIME::Attrs, which does
 what Achim suggests... so you could say:
@@ -851,9 +816,9 @@ We could even make it a superclass of MIME::Head: that way, MIME::Head
 would have to implement its interface, I<and> allow itself to be
 initiallized from a MIME::Attrs object.
 
-However, when you read RFC 2045, you begin to see how much MIME information
+However, when you read RFC-1521, you begin to see how much MIME information
 is organized by its presence in particular fields.  I imagine that we'd
-begin to mirror the structure of RFC 2045 fields and subfields to such
+begin to mirror the structure of RFC-1521 fields and subfields to such 
 a degree that this might not give us a tremendous gain over just
 having MIME::Head.
 
@@ -899,7 +864,6 @@ multiple times... and the different occurences had to be retrievable.
 =head1 AUTHOR
 
 Eryq (F<eryq@zeegee.com>), ZeeGee Software Inc (F<http://www.zeegee.com>).
-David F. Skoll (dfs@roaringpenguin.com) http://www.roaringpenguin.com
 
 All rights reserved.  This program is free software; you can redistribute 
 it and/or modify it under the same terms as Perl itself.
@@ -910,7 +874,7 @@ Lee E. Brotzman, Advanced Data Solutions.
 
 =head1 VERSION
 
-$Revision$ $Date$
+$Revision: 5.105 $ $Date: 2000/05/23 05:36:19 $
 
 =cut
 
